@@ -1,21 +1,21 @@
 package App::Sequence;
 use Simo;
 
-our $VERSION = '0.03_05';
+our $VERSION = '0.0201';
 
 use Carp;
-use Encode;
+use FindBin;
 
-### accessors( by Simo )
+### accessors
 
 # config file list
-sub conf_files{ ac
+sub conf_files{ ac 
     default => [],
     filter => \&_to_array_ref,
     trigger => \&_update_confs
 }
 # trigger method when conf_file is set
-sub _update_confs{ $_->confs( $_->_rearrange_conf( $_->conf_files ) ) }
+sub _update_confs{ $_->confs( _rearrange_conf( $_->conf_files ) ) }
 
 # config list
 sub confs{ ac default => [], filter => \&_to_array_ref };
@@ -28,17 +28,13 @@ sub sequence_files{ ac
 }
 
 # trigger method when sequence_files is set
-sub _update_sequences{ $_->sequences( $_->_rearrange_sequence( $_->sequence_files ) ) }
+sub _update_sequences{ $_->sequences( _rearrange_sequence( $_->sequence_files ) ) }
 
 # sequence list
 sub sequences{ ac default => [], filter => \&_to_array_ref }
 
 # module file list
-sub module_files{ ac 
-    default => [],
-    filter => \&_to_array_ref,
-    trigger =>  sub{ $_->_import_module( $_->module_files ) }
-}
+sub module_files{ ac default => [], filter => \&_to_array_ref }
 
 # retrun value
 sub r{ ac default => {} }
@@ -46,31 +42,39 @@ sub r{ ac default => {} }
 # @ARGV
 sub argv{ ac default => [], filter => \&_to_array_ref }
 
-# convert to array ref
-sub _to_array_ref{ ref eq 'ARRAY' ? $_ : [ $_ ] }
+# callback run
 
+sub _to_array_ref{ ref eq 'ARRAY' ? $_ : [ $_ ] }
 
 ### method
 
-# new is automaticaly created by Simo
+# new
+sub new{
+    my $self = shift->SUPER::new( @_ );
+    
+    _import_module( $self->module_files );
+    return $self;
+}
 
 # create object from @ARGV
 sub create_from_argv{
     my $self = shift->SUPER::new;
 
-    my $argv = $self->_rearrange_argv( @ARGV );
+    my $argv = _rearrange_argv( @ARGV );
     
     $self->conf_files( $argv->{ conf_files } );
     $self->sequence_files( $argv->{ sequence_files } );
     $self->module_files( $argv->{ module_files } );
     
+    _import_module( $self->module_files );
+    
     return $self;
 }
 
 # .pm files import
 sub _import_module{
-    my ( $self, $module_files ) = @_;
-
+    my $module_files = shift;
+    my $self = shift;
     use lib '.';
     
     foreach my $module_file ( @{ $module_files } ){
@@ -89,33 +93,33 @@ sub run{
     foreach my $conf ( @{ $self->confs } ){
         foreach my $sequence ( @{ $self->sequences } ){
             my $ret = {};
-            $self->_run_sequence( $sequence, $conf, $ret );
+            _run_sequence( $sequence, $conf, $ret );
         }
     }
 }
 
 # run each sequence
 sub _run_sequence{
-    my ( $self, $sequence, $conf, $ret ) = @_;
+    my ( $sequence, $conf, $ret ) = @_;
     foreach my $func_info ( @{ $sequence } ){
-        $self->_run_function( $func_info, $conf, $ret );
+        _run_function( $func_info, $conf, $ret );
     }
 }
 
 # run each function 
 sub _run_function{
-    my ( $self, $func_info, $conf, $ret ) = @_;
+    my ( $func_info, $conf, $ret ) = @_;
     my $func_name = $func_info->{ package } . '::' . $func_info->{ name };
     
     my @args;
     foreach my $arg ( @{ $func_info->{ args } } ){
-        my $val = $self->_parse_string_data( $arg, $conf, $ret );
+        my $val = _parse_string_data( $arg, $conf, $ret );
         push @args, $val;
         carp "$arg is undef value" if !defined( $val );
     }
     
-    my $ret_key = $func_info->{ ret };
-    if( $ret_key && $ret_key =~ /^r\.(.+)/ ){
+    my $ret_key;
+    if( $func_info->{ ret } =~ /^r\.(.+)/ ){
         $ret_key = $1;
     }
     
@@ -123,18 +127,19 @@ sub _run_function{
         no strict 'refs';
         my $ret_val = $func_name->( @args );
         
-        if( $ret_key && $ret_key =~ /^stdout$/ ){
-            print "$ret_val";
-        }
-        elsif( $ret_key ){
+        if( $ret_key ){
             $ret->{ $ret_key } = $ret_val;
         }
+        
+        if( $func_info->{ ret } =~ /^stdout$/ ){
+            print "$ret_val";
+        }
     }
 }
 
 # parse string data structure( c.name, c.age, etc )
 sub _parse_string_data{
-    my ( $self, $arg, $conf, $ret ) = @_;
+    my ( $arg, $conf, $ret ) = @_;
     my $val;
     if( $arg =~ s/^c\.// ){
         my @keys = split /\./, $arg;
@@ -153,12 +158,12 @@ sub _parse_string_data{
 
 # rearrange @ARGV
 sub _rearrange_argv{
-    my ( $self, @argv ) = @_;
+    my @argv = @_;
 
     my $rearranged_argv = { sequence_files => [], module_files => [], conf_files => [] };
     
-    if( my $meta_file = $self->_meta_file_contain( @argv ) ){
-        @argv = $self->_parse_meta_file( $meta_file );
+    if( my $meta_file = _meta_file_contain( @argv ) ){
+        @argv = _parse_meta_file( $meta_file );
     }
     
     foreach my $arg ( @argv ){
@@ -171,18 +176,17 @@ sub _rearrange_argv{
         elsif( $arg =~ /\.csv$/ ||
                $arg =~ /\.ya?ml$/ ||
                $arg =~ /\.xml$/ ||
-               $arg =~ /\.ini$/ ||
-               $arg =~ /\.json$/ )
+               $arg =~ /\.ini$/ )
         {
             push @{ $rearranged_argv->{ conf_files } }, $arg;
         }
         else{
-            croak "'$arg' is invalid param. param must be in ( .as .pm .csv .yaml .yml .xml .ini .json )";
+            croak "'$arg' is invalid param. param must be in ( .as .pm .csv .yaml .yml .xml .ini )";
         }
     }
     
     croak ".as file must be passed" unless @{ $rearranged_argv->{ sequence_files } };
-    croak "config file( .csv .yaml .yml .xml .ini .json ) must be passed"
+    croak "config file( .csv .yaml .yml .xml .ini ) must be passed"
         unless @{ $rearranged_argv->{ conf_files } };
     
     return $rearranged_argv;
@@ -190,7 +194,7 @@ sub _rearrange_argv{
 
 # whether array contain meta file( .meta )
 sub _meta_file_contain{
-    my ( $self, @argv ) = @_;
+    my @argv = @_;
     my $meta_file = ( grep { /\.meta$/ } @argv )[0];
     
     carp "Only first meta file $meta_file is received. Other arguments is ignored."
@@ -201,7 +205,7 @@ sub _meta_file_contain{
 
 # parse meta file, and convert @argv
 sub _parse_meta_file{
-    my ( $self, $file ) = @_;
+    my $file = shift;
     
     open my $fh, "<", $file
         or croak "Cannot open $file: $!";
@@ -231,7 +235,7 @@ sub _parse_meta_file{
 
 # parse .as file line
 sub _parse_func_expression{
-    my ( $self, $exp ) = @_;
+    my $exp = shift;
     my $original_exp = $exp;
     
     my $func_info = { package => 'main', name => undef, args => [], ret => undef };
@@ -288,7 +292,7 @@ sub _parse_func_expression{
 
 # parse sequence file and convert to sequence data.
 sub _rearrange_sequence{
-    my ( $self, $files ) = @_;
+    my $files = shift;
     my $sequences = [];
     
     foreach my $file ( @{ $files } ){
@@ -301,7 +305,7 @@ sub _rearrange_sequence{
             $line =~ s/\x0D\x0A|\x0D|\x0A/\n/g;
             chomp $line;
             
-            my $func_info = eval{ $self->_parse_func_expression( $line ) };
+            my $func_info = eval{ _parse_func_expression( $line ) };
             croak "$file line $. : $@" if( $@ );
             
             push @{ $sequence }, $func_info;
@@ -313,7 +317,7 @@ sub _rearrange_sequence{
 
 # parse many type config file, and convert hash ref.
 sub _rearrange_conf{
-    my ( $self, $conf ) = @_;
+    my $conf = shift;
     
     # convert array ref
     my $confs = ref $conf eq 'ARRAY' ? $conf : [ $conf ];
@@ -327,9 +331,14 @@ sub _rearrange_conf{
             $rearranged_conf = [ $conf ];
         }
         elsif( $conf =~ /\.xml$/ ){
-            $rearranged_conf = $self->_parse_xml( $conf );
+            require XML::Simple;
+            my $parser = XML::Simple->new;
+            eval{
+                $rearranged_conf =  $parser->XMLin( $conf );
+            };
+            croak $@ if $@;
         }
-        elsif( $conf =~ /\.ya?ml$/ ){
+        elsif( $conf =~ /\.yml$/ ){
             require YAML;
             eval{
                 $rearranged_conf = YAML::LoadFile( $conf );
@@ -340,16 +349,13 @@ sub _rearrange_conf{
             require Config::Tiny;
             my $ct = Config::Tiny->new;
             my $tiny_obj = $ct->read( $conf );
-            
+
             croak $ct->errstr unless $tiny_obj;
             $rearranged_conf = {};
             %{ $rearranged_conf } = %{ $tiny_obj };
         }
         elsif( $conf =~ /\.csv$/ ){
-            $rearranged_conf = $self->_parse_csv( $conf );
-        }
-        elsif( $conf =~ /\.json$/ ){
-            $rearranged_conf = $self->_parse_json( $conf );
+            $rearranged_conf = _parse_csv( $conf );
         }
         else{
             croak "$conf is unacceptable as conf setting";
@@ -365,29 +371,26 @@ sub _rearrange_conf{
 
 # csv file arrange
 sub _parse_csv{
-    my ( $self, $conf, $charset ) = @_;
-    $charset ||= 'utf8';
+    my $conf = shift;
+    require Text::CSV;
+    my $parser = Text::CSV->new({ binary => 1 });
     
     open my $fh, "<", $conf
-        or croak "Cannot open $conf: $!";
-    
-    require Text::CSV;
-    # my $parser = Text::CSV->new({ binary => 1 });
-    my $parser = Text::CSV->new;
+        or croak "Cannnot open $conf: $!";
     
     my $is_first_line = 1;
     my @header;
     my $rearranged_confs = [];
     while( my $line = <$fh> ){
-        $line = decode( 'utf8', $line );
         
         $line =~ s/\x0D\x0A|\x0D|\x0A/\n/g;
         chomp $line;
         
         next if $line =~ /^$/;
         
+        $line =~ s/(\r|\n)//g;
         $parser->parse( $line );
-        
+
         if( !$parser->status ){
             croak $parser->error_diag . ': ' . $parser->error_input;
         }
@@ -406,52 +409,16 @@ sub _parse_csv{
             push @{ $rearranged_confs }, $rearranged_conf;
         }
     }
-    close $fh;
-    
     return $rearranged_confs;
 }
 
-sub _parse_xml{
-    my ( $self, $conf ) = @_;
-    
-    require XML::Simple;
-    my $parser = XML::Simple->new;
-    my $rearranged_conf;
-    
-    croak "File '$conf' not exists" unless -f $conf;
-    
-    eval{ $rearranged_conf =  $parser->XMLin( $conf ) };
-    croak "File '$conf': $@" if $@;
-    
-    return $rearranged_conf;
-}
-
-sub _parse_json{
-    my ( $self, $conf, $charset ) = @_;
-    $charset ||= 'utf8';
-    
-    open my $fh, "<", $conf
-        or croak "Cannot open $conf: $!";
-    
-    my $content = do{ local $/; <$fh> }
-        or croak "Cannot read $conf: $!";
-    
-    $content = decode( $charset, $content );
-    
-    require JSON;
-    my $rearranged_conf = JSON::from_json( $content );
-    
-    close $fh;
-    return $rearranged_conf;
-}
-
 =head1 NAME
 
 App::Sequence - pluggable subroutine engine.
 
 =head1 VERSION
 
-Version 0.03_05
+Version 0.0201
 
 This version is alpha version. It is experimental stage.
 I have many works yet( charctor set, error handling, log outputting, some bugs )
@@ -499,8 +466,6 @@ apseq script receive three type of files.
 
 apseq script receive three type of file, and execute subroutines.
 
-File must be written by utf8.
-
 =head1 Three type of file
 
 =head2 Sequence file
